@@ -40,15 +40,15 @@ from shlex import split as shlex_split
 import shutil
 from time import mktime as time_mktime
 from datetime import datetime
-from typing import Dict, Union, TextIO, Set
+from typing import Dict, TextIO, Set, Optional
 from GPUmodules import __required_pversion__, __required_kversion__
+from GPUmodules.RegexPatterns import RegexPatterns
 
 LOGGER = logging.getLogger('gpu-utils')
 
 
 class GutConst:
-    """
-    GPU Utils constants used throughout the project.
+    """ GPU Utils constants used throughout the project.
     """
     # Private class variables
     _verified_distros: Set[str] = {'Debian', 'Ubuntu', 'Neon', 'Gentoo', 'Arch', 'Devuan'}
@@ -85,43 +85,17 @@ class GutConst:
                                      'green':     '\033[32m',
                                      'red':       '\033[31m',
                                      # Named formats
-                                     'amd':       '\033[1;37;41m',
+                                     'AMD':       '\033[1;37;41m',
                                      'error':     '\033[1;37;41m',
                                      'ok':        '\033[1;37;42m',
-                                     'nvidia':    '\033[1;30;42m',
+                                     'NVIDIA':    '\033[1;30;42m',
                                      'warn':      '\033[1;30;43m',
-                                     'intel':     '\033[1;37;44m',
+                                     'INTEL':     '\033[1;37;44m',
                                      'other':     '\033[1;37;45m',
                                      'label':     '\033[1;37;46m',
                                      'reset':     '\033[0;0;0m'}
 
-    PATTERNS = {'HEXRGB':       re.compile(r'^#[\da-fA-F]{6}'),
-                'PCIIID_L0':    re.compile(r'^[\da-fA-F]{4}.*'),
-                'PCIIID_L1':    re.compile(r'^\t[\da-fA-F]{4}.*'),
-                'PCIIID_L2':    re.compile(r'^\t\t[\da-fA-F]{4}.*'),
-                'END_IN_ALPHA': re.compile(r'[a-zA-Z]+$'),
-                'ALPHA':        re.compile(r'[a-zA-Z]+'),
-                'AMD_FEATURES': re.compile(r'^(Current pp)*\s*:*\s*features:*\s+', re.IGNORECASE),
-                'AMD_GPU':      re.compile(r'(AMD|ATI)'),
-                'NV_GPU':       re.compile(r'NVIDIA', re.IGNORECASE),
-                'INTC_GPU':     re.compile(r'INTEL'),
-                'ASPD_GPU':     re.compile(r'ASPEED', re.IGNORECASE),
-                'MTRX_GPU':     re.compile(r'MATROX', re.IGNORECASE),
-                'InputLabelX':  re.compile(r'[a-zA-Z]*(\d|\*)_(input|label)'),  # 'freq*_input'
-                'MHz':          re.compile(r'M[Hh]z'),
-                'PPM_CHK':      re.compile(r'[*].*'),
-                'PCI_GPU':      re.compile(r'(VGA|3D|Display)', re.IGNORECASE),
-                'NOT_PCI_GPU':  re.compile(r'Non-?VGA', re.IGNORECASE),
-                'PCI_ADD':      re.compile(r'^(([\da-fA-F]{4}:)?[\da-fA-F]{2}:[\da-fA-F]{2}.[\da-fA-F])'),
-                'PCI_ADD_LONG': re.compile(r'^([\da-fA-F]{4}:[\da-fA-F]{2}:[\da-fA-F]{2}.[\da-fA-F])'),
-                'PCI_ADD_SHRT': re.compile(r'^([\da-fA-F]{2}:[\da-fA-F]{2}.[\da-fA-F])'),
-                'PPM_NOTCHK':   re.compile(r'\s+'),
-                'VALID_PS_STR': re.compile(r'\d+(\s\d)*'),
-                'IS_FLOAT':     re.compile(r'[-+]?\d*\.?\d+|[-+]?\d+'),
-                'DIGITS':       re.compile(r'^\d+\d*$'),
-                'VAL_ITEM':     re.compile(r'.*_val$'),
-                'GPU_GENERIC':  re.compile(r'(^\s|intel|amd|nvidia|amd/ati|ati|radeon|\[|])', re.IGNORECASE),
-                'GPUMEMTYPE':   re.compile(r'^mem_(gtt|vram)_.*')}
+    PATTERNS = RegexPatterns()
 
     featuremask: str = '/sys/module/amdgpu/parameters/ppfeaturemask'
     card_root: str = '/sys/class/drm/'
@@ -131,10 +105,10 @@ class GutConst:
     TIME_FORMAT: str = '%d-%b-%Y %H:%M:%S'
 
     def __init__(self):
-        self.calling_program = ''
-        self.args: Union[argparse.Namespace, None] = None
+        self.calling_program: str = ''
+        self.args: Optional[argparse.Namespace] = None
         self.repository_path: str = self._repository_path
-        self.install_type: Union[str, None] = None
+        self.install_type: Optional[str] = None
         self.package_path: str = inspect.getfile(inspect.currentframe())
 
         if 'dist-packages' in self.package_path: self.install_type = 'debian'
@@ -152,9 +126,9 @@ class GutConst:
             print('Error: Invalid pciid path')
             self.sys_pciid = None
 
-        self.distro: Dict[str, Union[str, None]] = {'Distributor': None, 'Description': None}
-        self.amdfeaturemask: Union[int, None] = None
-        self.log_file_ptr: Union[TextIO, None] = None
+        self.distro: Dict[str, Optional[str]] = {'Distributor': None, 'Description': None}
+        self.amdfeaturemask: Optional[int] = None
+        self.log_file_ptr: Optional[TextIO] = None
 
         # From args
         self.no_markup: bool = False
@@ -173,11 +147,11 @@ class GutConst:
         # Time
         self.ltz: datetime.tzinfo = datetime.utcnow().astimezone().tzinfo
         # Command access
-        self.cmd_lsb_release: Union[str, None] = None
-        self.cmd_lspci: Union[str, None] = None
-        self.cmd_clinfo: Union[str, None] = None
-        self.cmd_dpkg: Union[str, None] = None
-        self.cmd_nvidia_smi: Union[str, None] = None
+        self.cmd_lsb_release: Optional[str] = None
+        self.cmd_lspci: Optional[str] = None
+        self.cmd_clinfo: Optional[str] = None
+        self.cmd_dpkg: Optional[str] = None
+        self.cmd_nvidia_smi: Optional[str] = None
 
         # base_prefix = getattr(sys, "base_prefix", None) or getattr(sys, "real_prefix", None) or sys.prefix
         # print('base_prefix: {}, sys_prefix: {}'.format(base_prefix, sys.prefix))
@@ -185,8 +159,7 @@ class GutConst:
         #     sys.path.append('/usr/lib/python3/dist-packages')
 
     def set_args(self, args: argparse.Namespace, program_name: str = '') -> None:
-        """
-        Set arguments for the give args object.
+        """ Set arguments for the give args object.
 
         :param args: The object return by args parser.
         :param program_name: The name of the calling program.
@@ -238,8 +211,7 @@ class GutConst:
 
     @staticmethod
     def now(ltz: bool = False) -> datetime:
-        """
-        Get the current datetime object.
+        """ Get the current datetime object.
 
         :param ltz: Flag to get local time instead of UTC
         :return: datetime obj of current time
@@ -248,8 +220,7 @@ class GutConst:
 
     @staticmethod
     def utc2local(utc: datetime) -> datetime:
-        """
-        Return local time for given UTC time.
+        """ Return local time for given UTC time.
 
         :param utc: Time for UTC
         :return: Time for local time zone
@@ -260,9 +231,8 @@ class GutConst:
         return utc + offset
 
     def process_message(self, message: str, log_flag: bool = False) -> None:
-        """
-        For given message, print to stderr and/or LOGGER depending on command line options and
-        the value of log_flag.
+        """ For given message, print to stderr and/or LOGGER depending on command line options and
+            the value of log_flag.
 
         :param message: A string containing the message to be processed.
         :param log_flag:  If True, write to LOGGER.
@@ -272,8 +242,7 @@ class GutConst:
         if log_flag: LOGGER.debug(message)
 
     def read_amdfeaturemask(self) -> int:
-        """
-        Read and return the amdfeaturemask as an int.
+        """ Read and return the amdfeaturemask as an int.
 
         :return: AMD Feature Mask
         """
@@ -292,8 +261,7 @@ class GutConst:
         return self.amdfeaturemask
 
     def check_env(self) -> int:
-        """
-        Check the compatibility of the user environment.
+        """ Check the compatibility of the user environment.
 
         :return: Return status: ok=0, python issue= -1, kernel issue= -2, command issue= -3
         """
@@ -309,13 +277,16 @@ class GutConst:
 
         # Check Linux Kernel version
         current_kversion_str = release()
-        current_kversion = tuple([int(x) for x in re.sub('-.*', '', current_kversion_str).split('.')])
         LOGGER.debug('Using Linux Kernel: %s', current_kversion_str)
-        if current_kversion < __required_kversion__:
-            print('Using Linux Kernel {}, but {} requires > {}.{}.'.format(
-                current_kversion_str, __program_name__, __required_kversion__[0],
-                __required_kversion__[1]), file=sys.stderr)
-            return -2
+        try:
+            current_kversion = tuple([int(x) for x in re.sub('(-.*)|([a-zA-Z]+)', '', current_kversion_str).split('.')])
+            if current_kversion < __required_kversion__:
+                print('Using Linux Kernel {}, but {} requires > {}.{}.'.format(
+                    current_kversion_str, __program_name__, __required_kversion__[0],
+                    __required_kversion__[1]), file=sys.stderr)
+                return -2
+        except ValueError:
+            print('Linux Kernel version check skipped.')
 
         # Check Linux Init Type
         init_type = 'Unknown'
@@ -385,8 +356,7 @@ class GutConst:
         return 0
 
     def read_amd_driver_version(self) -> bool:
-        """
-        Read the AMD driver version and store in GutConst object.
+        """ Read the AMD driver version and store in GutConst object.
 
         :return: True on success.
         """
@@ -402,8 +372,7 @@ class GutConst:
         return False
 
     def read_amd_driver_version_gentoo(self) -> bool:
-        """
-        Read the AMD driver version and store in GutConst object.
+        """ Read the AMD driver version and store in GutConst object.
 
         :return: True if successful
         """
@@ -428,8 +397,7 @@ class GutConst:
         return False
 
     def read_amd_driver_version_arch(self) -> bool:
-        """
-        Read the AMD driver version and store in GutConst object.
+        """ Read the AMD driver version and store in GutConst object.
 
         :return: True if successful
         """
@@ -451,8 +419,7 @@ class GutConst:
         return False
 
     def read_amd_driver_version_debian(self) -> bool:
-        """
-        Read the AMD driver version and store in GutConst object.
+        """ Read the AMD driver version and store in GutConst object.
 
         :return: True if successful
         """
